@@ -15,7 +15,7 @@ use 5.010;
 ### EVENT FIRE OBJECTS ###
 ##########################
 
-our $VERSION = '5.431';
+our $VERSION = '5.47';
 our $events  = $Evented::Object::events;
 our $props   = $Evented::Object::props;
 
@@ -155,6 +155,11 @@ sub object {
     shift->{$props}{object};
 }
 
+# returns the exception from 'safe' option, if any.
+sub exception {
+    shift->{$props}{exception};
+}
+
 # internal use only.
 # returns an array of the callbacks to come.
 sub _pending_callbacks {
